//********************************************************************************************
//*         Name:  calc1.c
//*
//*       Author:  Daniel F. Ramirez
//* 
//*          Date:  10/21/2001
//* 
//*   Description:  
//* 
//* 		This is PIC18C452 floating point application that runs on a DIY calculator. 
//* The application performs polish string conversions from algebraic expressions and then 
//* evaluates them. It will run on the PIC18C452 with an attached keyboard and LED or LCD
//* display.  It includes the basic arithmetic operators, "+","-","*", and "/".  
//*
//* 	The calculator supports I/O via the serial port (USART), and the Hex keypad.  It supports
//* LED or LCD displays.
//* 
//* 		Future upgrades to the software include the addition of conversions (Hex, Decimal, Binary),
//* trigonometric, scientific, and financial functions.  Other improvements include simulating
//* an HP programmable calculator with RPN. These new features may require additional SRAM, 
//* FLASH or EEPROM.
//* 
//*********************************************************************************************

  
// Initialize Calculator compilation flags here

//#define SINGLE_PRECISION                               // Include Single Precision code

//#define TEST                                           // Test code

#ifdef PIC18C452

// Include function headers here

  
/*
#ifdef PIC18C252     
#include <p18c252.h>													// Register definitions for PIC18C252
#endif

#ifdef PIC18C452     
#include <p18c452.h>													// Register definitions for PIC18C452
#endif
*/
#endif
  
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
//#include <i2c.h>												  // I2C library function 

#include "flags.h"                                      	// Global flags used for MPSIM debugging 

#ifdef PIC18C452

#include "timeutil.h"                                   // Header file for timer utilities
#else
#include <stdio.h>                                      // Header file for stdio file
#endif
                        
//#ifdef SERIAL_DRIVER
#include "stdio.h"                                      // Header file for stdio file
//#endif

//#ifdef KEYPAD_DRIVER
#include "keypad.h"                                     // Header file for keypad Driver
//#endif

#ifdef LED_DRIVER
#include "max7219.h"    										  // Header file for Max7219 LED Display Drivers
#endif

#ifdef LCD_DRIVER
#include "lcd.h"    										  			// Header file for LCD Display Drivers
#endif

#define TRUE 1
#define FALSE 0

#define BufferSize 15			// Floating point buffer size

// Declare global variables here

 
static char af1[BufferSize];    // Holds ascii representation of an IEEE floating point number for testing

char Period[] = ".";
char Plus[] = "+";
char Minus[] = "-";
char Blank[] = " ";
char E[] = "E";

char Operators[] = {'+','-','*','/','.','=',0};          // Math operators 

//extern char Operators[];                             // Math operators declared in keypad.c
char NumericChars[] = {'.','E',0};                   // Numeric characters

char TheOperator;        // Current operator function key: "+","-","*","/"
char TheEqualSign;       // Equal sign, "="

double Argument1;
double Argument2;
double Result;
char Buffer[BufferSize];         // Floating point Buffer
 
byte KeyValue; 

 
// Power of 10 lookup tables used by the pow10 function

#ifdef PIC18C452
//     10 **                 0     1      2      3     4       5      6      7      8      9      10
const rom double PlusPowerOf10[11] = {1.0, 1.0E1, 1.0E2, 1.0E3, 1.0E4, 1.0E5, 1.0E6, 1.0E7, 1.0E8, 1.0E9, 1.0E10 };
const rom double MinusPowerOf10[11] = {1.0, 1.0E-1, 1.0E-2, 1.0E-3, 1.0E-4, 1.0E-5, 1.0E-6, 1.0E-7, 1.0E-8, 1.0E-9, 1.0E10 };
#else
//     10 **                 0     1      2      3     4       5      6      7      8      9      10
const double PlusPowerOf10[11] = {1.0, 1.0E1, 1.0E2, 1.0E3, 1.0E4, 1.0E5, 1.0E6, 1.0E7, 1.0E8, 1.0E9, 1.0E10 };
const double MinusPowerOf10[11] = {1.0, 1.0E-1, 1.0E-2, 1.0E-3, 1.0E-4, 1.0E-5, 1.0E-6, 1.0E-7, 1.0E-8, 1.0E-9, 1.0E-10 };
#endif

// Declare function headers here

void main(void);
double Calculator(double Argument1, char Operator, double Argument2);

// Floating point conversion functions

void Normalize(double *Value, int *Exponent);
unsigned long NormalizeFraction(double FractionalPart, int NumberOfDigits);

#ifdef SINGLE_PRECISION
void FloatToAscii(float Number, char * s);
void AsciiToFloat(float Number, char * s);
#endif
void DoubleToAscii(double Number, char * s);
void AsciiToDouble(double Number, char * s);
int Member(char Ch, char * Set);

float GetFloat(void);
double GetDouble(void);

double log10(double x);
double pow10(int x);
char GetKey(void);

void main (void)
{

	byte Ch;
	
#ifdef PIC18C452
     
#ifdef SERIAL_DRIVER 

    // Configure USART for 20 MHz, 9600 Baud, 8-Bits, No parity, Asynchronous
   
    InitializeUsart();
    
	// Initialize Serial LCD Display here
	
	//printf("\r8-N-1");  
	 
   
#endif

    // Setup timers 

    SetupTimers();  

 
#ifdef LCD_DRIVER
 		InitializeLCD();
		//lcd_test_1();
		//lcd_test_2();
	   //lcd_clear();						// clear LCD, home cursor

#endif

	printf("DIY Calculator");    
/*	
	printf("Delay 5 seconds ...\r\n");
	pause(5000);
	printf("End of 5 seconds.\r\n");
*/


#ifdef LED_DRIVER
		printf("trace 2:\r\n");
		InitializeMAX7219();
#endif


#ifdef KEYPAD_DRIVER

  // Initialize keypad, Setup timers, configure ports and initialize variables
		printf("trace 3:\r\n");

   InitializeKeypad();  
#endif




#endif

/*

    printf("PIC18C452 Start KeyScan Test ... \n\n");
  
again:
    
    KeyValue = GetKey() - 48;					// Convert the key value from ASCII to binary
  
    // Send the scanned key to the host

    printf("Key pressed = ");
    dec(KeyValue);
    printf("\r\n");

    goto again;

*/

	
	// Get floating point arguments from the keyboard or keypad and compute the desired operation
	// using the  Calculator function.
	
	
	while (1)
	{
#ifdef LCD_DRIVER
	    lcd_clear();						// clear LCD, home cursor	    
		puts("> ");			      	    // Display calculator prompt
#endif
 
		// Get first floating point argument and the corresponding argument

#ifdef SERIAL_DRIVER
		printf("\r\n> ");			    	// Display calculator prompt
		
		//printf("Enter Argument 1: ");
#endif

		Argument1 = GetDouble();
		
#ifdef SERIAL_DRIVER
		//printf("\r\n");
#endif

		// Get second floating point argument and the equal sign

#ifdef SERIAL_DRIVER
		//printf("Enter Argument 2: ");
#endif
		Argument2 = GetDouble();
		
#ifdef SERIAL_DRIVER
		//printf("\r\n");
#endif

		// Compute the desired operation

		Result = Calculator(Argument1, TheOperator, Argument2);
		//MyResult = (int) Result;

#ifdef SERIAL_DRIVER
		//printf("%f + %f = %f \r\n", Argument1, Argument2, Result);
#endif

		// Convert argument1 from float to ascii for display to the monitor, LED or LCD.

		DoubleToAscii(Argument1, af1);
		
#ifdef LED_DRIVER	
		// Send output to LED		
		MaxPrint(af1);
#endif

#ifdef SERIAL_DRIVER	

		// Send output to Serial Port
/*		
		puts(af1);
		printf(" ");
		putchar(TheOperator);
		printf(" ");
*/
#endif

		// Convert argument1 from float to ascii for display to the monitor, LED or LCD.

		DoubleToAscii(Argument2, af1);
		
#ifdef LED_DRIVER	
		// Send output to LED		
		MaxPrint(af1);
#endif		

#ifdef SERIAL_DRIVER
		// Send argument 2 to Serial Port
/*
		puts(af1);
		printf(" = ");
*/
#endif		
		
		DoubleToAscii(Result, af1);
		
#ifdef LED_DRIVER	
		// Send result to LED		
		MaxPrint(af1);
#endif			
		
#ifdef SERIAL_DRIVER
		// Send result to Serial Port
		puts(af1);
		//printf("\r\n");
#endif
		
		
#ifdef SERIAL_DRIVER
		// Hit any  key to continue
		Ch = getchar();                   // Serial Input   
#endif
				
#ifdef KEYPAD_DRIVER
		// Hit any  key to continue
		Ch = GetKey();                      // Keypad Input    
#endif


	}


//****************************************************************************

#ifdef TEST

	// Test code 
   // Test strcat function

   strcat(af1, Plus);
   strcat(af1, Minus);
   strcat(af1, Period);
   strcat(af1, E);

	Argument1 = 323.1123E+5;
    DoubleToAscii(Argument1, af1);

	Argument1 = -12345.6789;
    DoubleToAscii(Argument1, af1);
	

	Argument1 = pow10(3);
   Argument2 = log10(4500.0);
   Argument2 = log10(10000.0);

	Argument1 = -12345.6789;
    DoubleToAscii(Argument1, af1);
	printf("Argument1 = %lf  af1 = %s\r\n", Argument1, af1);

	Argument1 = 323.1123E+5;
    DoubleToAscii(Argument1, af1);
	printf("Argument1 = %lf  af1 = %s\r\n", Argument1, af1);

	Argument1 = 1234.244E+8;
    DoubleToAscii(Argument1, af1);
	printf("Argument1 = %lf  af1 = %s\r\n", Argument1, af1);

	Argument1 = -31415.023E+3;
    DoubleToAscii(Argument1, af1);
	printf("Argument1 = %lf  af1 = %s\r\n", Argument1, af1);

	Argument1 = -7654.0E-11;
    DoubleToAscii(Argument1, af1);
	printf("Argument1 = %lf  af1 = %s\r\n", Argument1, af1);


   
    Argument1 = 3.5;
    Argument2 = 6.5;
 
	// Invoke calculator for test

	// Test addition

	Result = Calculator(Argument1, '+', Argument2);
	//printf("%f + %f = %f \r\n", Argument1, Argument2, Result);


	// Convert argument1 from float to ascii

	DoubleToAscii(Argument1, af1);
   puts(af1);
   printf(" + ");
	DoubleToAscii(Argument2, af1);
   puts(af1);
   printf(" = ");
	DoubleToAscii(Result, af1);
   puts(af1);
   printf("\r\n");

	// Test subtraction

	Result = Calculator(Argument1, '-', Argument2);
	//printf("%f - %f = %f \r\n", Argument1, Argument2, Result);
	DoubleToAscii(Argument1, af1);
   puts(af1);
   printf(" - ");
	DoubleToAscii(Argument2, af1);
   puts(af1);
   printf(" = ");
	DoubleToAscii(Result, af1);
   puts(af1);
   printf("\r\n");

	// Test multiplication

	Result = Calculator(Argument1, '*', Argument2);
	//printf("%f * %f = %f \r\n", Argument1, Argument2, Result);
	DoubleToAscii(Argument1, af1);
   puts(af1);
   printf(" * ");
	DoubleToAscii(Argument2, af1);
   puts(af1);
   printf(" = ");
	DoubleToAscii(Result, af1);
   puts(af1);
   printf("\r\n");

	// Test division

	Result = Calculator(Argument1, '/', Argument2);
	//printf("%f / %f = %f \r\n", Argument1, Argument2, Result);
	DoubleToAscii(Argument1, af1);
   puts(af1);
   printf(" / ");
	DoubleToAscii(Argument2, af1);
   puts(af1);
   printf(" = ");
	DoubleToAscii(Result, af1);
   string(af1);
 
   Ch = getch();			// Wait for extra operator keystroke before clearing the display
#endif
  
}

//******************************************************************************
//* Calculator - Simple calculator that performs double precision floating point
//* addition, subtraction, multiplication and division.  It does not currently
//* support levels of nesting parentheses for complicated mathematical expressions.
//******************************************************************************

double Calculator(double Argument1, char Operator, double Argument2)
{
	double TheResult;

	switch(Operator)
	{
		case '+':
		{
			TheResult = Argument1 + Argument2;
			break;
		}

		case '-':
		{
			TheResult = Argument1 - Argument2;
			break;
		}

		case '*':
		{
			TheResult = Argument1 * Argument2;
			break;
		}

		case '/':
		{
			TheResult = Argument1 / Argument2;
			break;
		}

		default:
		{
			TheResult = 0;
			break;
		}		

	}

	return (TheResult);

}

//******************************************************************************
//* Normalize - Normalizes the IEEE double precision floating point number in exponential 
//* notation so that only 1 digit appears to the left of the decimal point. The 
//* exponent is adjusted accordingly.  Used by printf to output floating point numbers.
//******************************************************************************

void Normalize(double *Value, int *Exponent)
{
	double TheNormalizedValue;
	int TheNormalizedExponent;
 
    int i;

	TheNormalizedValue = *Value;
   TheNormalizedExponent = *Exponent;

	if (TheNormalizedValue > 10.0)
	{
		while(TheNormalizedValue > 10.0)
		{
			TheNormalizedValue = TheNormalizedValue / 10.0;
			TheNormalizedExponent++;
			//printf("trace 1: TheNormalizedExponent = %d,  TheNormalizedValue = %lf ", TheNormalizedExponent, TheNormalizedValue);
		}
	}
	else
	if ((TheNormalizedValue > 0.0)  && (TheNormalizedValue < 1.0))
	{
		while((TheNormalizedValue > 0.0)  && (TheNormalizedValue < 1.0))
		{
			TheNormalizedValue = TheNormalizedValue * 10.0;
			TheNormalizedExponent--;
			//printf("trace 2: TheNormalizedExponent = %d,  TheNormalizedValue = %lf ", TheNormalizedExponent, TheNormalizedValue);
		}
	}

	*Value = TheNormalizedValue;
	*Exponent = TheNormalizedExponent;
 
}

//******************************************************************************
//* NormalizeFraction - Normalizes the fractional part of an IEEE double precision 
//* floating point number. Used by printf to output floating point numbers.
//******************************************************************************
 
unsigned long NormalizeFraction(double FractionalPart, int NumberOfDigits)
{
	unsigned long TheNormalizedPart;
   double x,y;

  	//TheNormalizedPart = (unsigned long) (pow(10.0, (double) (NumberOfDigits - 1)) * FractionalPart);
   TheNormalizedPart = (unsigned long) (pow10(NumberOfDigits - 1) * FractionalPart);
  
	return TheNormalizedPart;  
}

#ifdef SINGLE_PRECISION

//******************************************************************************
//* FloatToAscii - Converts an IEEE single precision floating point number from 
//* binary format to an ASCII string. Used by printf to output floating point numbers.
//******************************************************************************

void FloatToAscii(float Number, char * s)
{
	// Invoke DoubleToAscii for now

	DoubleToAscii((float)Number, s);
}

//******************************************************************************
//* AsciiToFloat - Converts an IEEE single precision floating point number from 
//* ASCII format to an IEEE float in binary format.  Used by the scanf function
//* to read floating point numbers.  
//******************************************************************************

void AsciiToFloat(float Number, char * s)
{
	// Invoke AsciiToDouble for now

	AsciiToDouble((float) Number, s);
}
#endif

//******************************************************************************
//* MantissaToAscii - Converts the mantissa part of an IEEE double precision
//* floating point number from binary format to an ASCII string.  Used by printf
//* to output floating point numbers.
//******************************************************************************

void MantissaToAscii(double Number, int Sign, char * s)
{
	unsigned long IntegralPart;              // Holds the floating point integral part
	unsigned long NormalizedPart;            // Holds the floating point normalized fractional part  
	double FractionalPart;					     // Holds the floating point fractional part

	int i;                                   // Index 
	int n;                                   // Number of digits in the mantissa
	char s1[BufferSize];                             // Temporary string
	char s2[BufferSize];                             // Temporary string

   
	IntegralPart = (unsigned long) (Number);
	FractionalPart = (double) (Number - (unsigned long) (Number)) ;

 	// Take care of the mantissa's sign

	if (Sign == -1)
	{
#ifdef PIC18C452
		strcpy(s, Minus);
#else
		strcpy(s, Minus);
#endif
	}
	else
	{
		strcpy(s,Blank);
	}
	// Convert integral part of the floating point number to ascii

#ifdef PIC18C452
	ultoa(IntegralPart, s1);
#else
	ultoa(IntegralPart, s1, 10);
#endif

	// Determine how many digits are in the Mantissa

	n = strlen(s1);

    //printf("n = %d \r\n",n);

	// Normalize the fractional part of the floating point number  

	NormalizedPart = NormalizeFraction(FractionalPart, (11 - n));

	// Convert fractional part of the floating point number to ascii (Number 
	// of digits remaining is (11 - n), including the decimal point. 

#ifdef PIC18C452
	ultoa(NormalizedPart, s2);
#else
	ultoa(NormalizedPart, s2, 10);
#endif

	//printf("IntegralPart = %ld \r\n", IntegralPart);
	//printf("FractionalPart = %lf \r\n", FractionalPart);
	//printf("NormalizedPart = %ld \r\n", NormalizedPart);
	//printf("Sign of mantissa = %d \r\n", Sign);

	//printf("s = !%s!",s);
	//printf("s1 = !%s!",s1);

	// Concatenate fractional part to integer part including the decimal point

	strcat(s, s1);
	strcat(s, Period);
	strcat(s, s2); 

}

//******************************************************************************
//* DoubleFloatToAscii - Converts an IEEE double precision floating point number 
//* from binary format to an ASCII string.  Used by printf to output
//* floating point numbers.
//******************************************************************************

void DoubleToAscii(double Number, char * s)
{
	unsigned long IntegralPart;              // Holds the floating point integral part
	unsigned long NormalizedPart;            // Holds the floating point normalized fractional part 
	unsigned long Mantissa;                  // Holds the floating point mantissa used for exponential notation 
   int SignOfMantissa;                      // Holds the sign of the floating point mantissa
   int SignOfExponent;                      // Holds the sign of the floating point exponent

	int Exponent;                            // Holds the floating point exponent 
	double FractionalPart;					     // Holds the floating point fractional part

	int i;                                   // Index 
	int n;                                   // Number of digits in the mantissa
	char s1[BufferSize];                             // Temporary string
	char s2[BufferSize];                             // Temporary string
 
 

	// Determine exponent

	Exponent = (int) log10(Number);

 	// Determine sign of mantissa and exponent

	if (Number >= 0.0)
	{
		SignOfMantissa = 1;
	}
	else
	{
		SignOfMantissa = -1;		
		Number = -Number;
	}


	if (Exponent >= 0)
	{
		SignOfExponent = 1;
	}
	else
	{
		SignOfExponent = -1;
 	}

	if ((Exponent >= 0) && (Exponent <= 10)) 
	{
		// Convert normal float number with no exponent (need to include the sign)

		MantissaToAscii(Number, SignOfMantissa, s);

	}
	else
	{
		// Normalized the float number using exponential notation

		Normalize(&Number, &Exponent);
/*
		printf("Mantissa = %ld \r\n", Number);
	   printf("Exponent = %d \r\n", Exponent);
		printf("Sign of Mantissa = %d \r\n", SignOfMantissa);
		printf("Sign of Exponent = %d \r\n", SignOfExponent);
*/

		// Convert the mantissa (Number) and the exponent to ascii.  Include characters for the 
		// mantissa and exponent signs, and the "E" for the exponent.

		// Convert mantissa to ascii

		MantissaToAscii(Number, SignOfMantissa, s1);

#ifdef PIC18C452

		// Convert exponent to ascii

		ultoa(Exponent, s2);

#else
		// Convert exponent to ascii

		ultoa(Exponent, s2, 10);
#endif

		// Take care of the mantissa's sign

		if (SignOfMantissa == -1)
		{
			strcpy(s, Minus);
		}
		else
		{
			strcpy(s,Blank);
		}
		
		// Concatanate mantissa

		strcat(s, s1);

		// Concatenate the exponent symbol

		strcat(s,E);

		// Take care of the exponent's sign

		if (SignOfMantissa == -1)
		{
			strcat(s, Minus);
		}
		else
		{
			strcat(s,Plus);
		}
		// Concatanate exponent

		strcat(s, s2);

	}
 
}

//******************************************************************************
//* AsciiToDouble - Converts an IEEE double precision floating point number from
//* ASCII format to an IEEE float in binary format. Used by the scanf function to
//* read long floating point
//* numbers.  
//******************************************************************************

void AsciiToDouble(double Number, char * s)
{
}

//******************************************************************************
//* log10 - Computes the integral log base 10 of argument, x, where x > 0.0, 
//* using a lookup table.
//******************************************************************************
double log10(double x)
{
	double TempValue;
   double TheLog;

	TempValue = x;
	TheLog = 0;

   
   if (TempValue > 0.0)
	{
		while (TempValue >= 10.0)
		{
			TempValue = TempValue / 10.0;
			TheLog++;
		} 
	}

	return TheLog;

}

//******************************************************************************
//* pow10 - Computes 10 raised to the integer power of argument, x, 
//* range -10 <= x <= 10, using a table lookup.
//******************************************************************************
double pow10(int x)
{
	double TempValue;
 
	if ((x >= 0) && (x <= 10))
	{
   		TempValue = PlusPowerOf10[x];
	}
	else
	{
   		TempValue = MinusPowerOf10[-x];
	}

	return TempValue;
}

//******************************************************************************
//* Member - Determines if a character, Ch, is an element of a given set.
//******************************************************************************
int Member(char Ch, char * Set)
{
	int i;
	int FoundFlag;

	FoundFlag =  FALSE;

	for (i = 0; i< strlen(Set); i++)
	{
		if (Ch == Set[i]) 
		{
			FoundFlag = TRUE;
			goto ExitLoop;
		}
	}
ExitLoop:
	return FoundFlag;
}


#ifdef SINGLE_PRECISION

//******************************************************************************
//* GetFloat - Scans a single precision floating point number from the keypad
//* or reads it from the serial port (USART). Also checks for the mantissa sign and 
//* checks for exponent the number is entered in exponential notation.  
//******************************************************************************
float GetFloat(void)
{	
	// Use GetDouble function for now

	return ((float) GetDouble());

}

#endif
										
//******************************************************************************
//* GetDouble - Scans a double precision floating point number from the keypad
//* or reads it from the serial port (USART). Also checks for the mantissa sign and 
//* checks for exponent the number is entered in exponential notation.  
//******************************************************************************
double GetDouble(void)
{													

	double DoubleFloatValue = 0.0;

   char Ch;				// Character 
	static int Count;				// Character count
	int Digit;				// Floating point digit (0..9)
	int i;					// Loop index

 

	Count = 0;
 	DoubleFloatValue = 0.0;   
	Ch = ' ';

	// Continue scanning for input until an operator: "+","-","*","/","=" is found

	//while (!Member(Ch, Operators))
	while (Member(Ch, Operators) == 0)
	{

		// Store the ASCII IEEE Floating Point number in a buffer use the 
		// operator key +,-,*,/,= as a delimiter.

		do
		{
			// Get a keystroke from the operator using the hex key pad or
			// the serial port (USART) and store it in the floating point work buffer.

#ifdef SERIAL_DRIVER

			//Ch = getchar();                   // Serial Input and Output 
			Ch = GetKey();								// Keypad Input and Serial Output  

#else

			Ch = GetKey();                      // Keypad Input and LED or LCD output  

#endif

			Buffer[Count++] = Ch;

 
		//} while (!Member(Ch, Operators) && !Member(Ch, NumericChars) && 
		//	    (Count < BufferSize));
		} while ((Member(Ch, Operators) == 0) && (Member(Ch, NumericChars) == 0) && 
			    (Count < BufferSize));
		Count = Count - 1;
		

		// Convert the Integral Part of an  IEEE Floating Point number

		for (i = 0; i < Count; i++) 
		{
 			// Get a digit from the floating point work buffer 

			Ch = Buffer[i];    
			Digit = Ch - 48;    // Convert to Binary
 
 			//if (!Member(Ch, Operators)) 
			if (Member(Ch, Operators) == 0) 
			{
				DoubleFloatValue = DoubleFloatValue + (double) Digit * pow10(Count-i-1);
			}
		
		} 
		
		// Check terminal character for fractional part of floating point number or for operator (if any)

		Ch = Buffer[Count];
  
		if (Ch == '.')
		{

			Count = 0;

			do
			{
				// Get a keystroke from the operator using the hex key pad or
				// the serial port (USART) and store it in the floating point work buffer.

#ifdef SERIAL_DRIVER

				//Ch = getchar();                   // Serial Input and Output 
				Ch = GetKey();								// Keypad Input and Serial Output  

#else

				Ch = GetKey();                      // Keypad Input and LED or LCD output  

#endif
				Buffer[Count++] = Ch;

 
			//} while (!Member(Ch, Operators) && !Member(Ch, NumericChars) && 
			//    (Count < BufferSize));
			} while ((Member(Ch, Operators) == 0) && (Member(Ch, NumericChars) == 0) && 
			    (Count < BufferSize));

			Count = Count - 1;

			// Convert the Fractional Part of an IEEE Floating Point number (if present)

			for (i = 0; i < Count; i++) 
			{
				// Get a keystroke from the floating point work buffer 

				Ch = Buffer[i];    
				Digit = Ch - 48;    // Convert to Binary
 
 				//if (!Member(Ch, Operators)) 
				if ((Member(Ch, Operators) == 0)) 
				{
					DoubleFloatValue = DoubleFloatValue + (double) Digit * pow10(-i-1);
				}
			
			} 

			// Check terminal character for operator (if any) or equal sign

			Ch = Buffer[Count];
 	
			if (Member(Ch, Operators))
			{
				if (Ch == '=')
				{
					TheEqualSign = Ch;
				}
				else
				{
					TheOperator = Ch;
				}
			}
	
		}
		else
		{
			// Check terminal character for operator (if any) or equal sign

			Ch = Buffer[Count];
 	
			if (Member(Ch, Operators))
			{
				if (Ch == '=')
				{
					TheEqualSign = Ch;
				}
				else
				{
					TheOperator = Ch;
				}
			}
		}

	}

	return DoubleFloatValue;
}

//******************************************************************************
//* GetKey - Scans a character from the keypad or reads it from the serial 
//* port (USART).    
//******************************************************************************
char GetKey(void)
{
	char TheKey;
	
	TheKey = get_byte();
 	putch(TheKey);							// Echo the character

 	return TheKey;
}

