//********************************************************************************
//*                                                                              *
//* Description:                                                                 *
//*                                                                              *
//*     Name:  lcd.h                                                             *
//*   Author:  Daniel F. Ramirez                                                 *
//*     Date:  02/05/2002                                                        *
//*                                                                              *
//* Description:                                                                 *
//*         This is an embedded realtime LCD Display application, that uses the  *
//* LCD library and a PIC18C452 micro-controller from Microchip Technology to    *
//* drive serial LCD displays.  It employs the shiftout and pulsout functions to *
//* communicate to the Serial LCD and uses reverse, itoa and right_justify       *
//* functions to format data.                                                    *
//*                                                                              *
//* Provides the following functions ported from :                               *
//* lcd.c Routines for sending data to a serial LCD                              *
//*  This application uses code from Mark Crosbie.  Thanks Mark.                 *
//*                                                                              *
//*  Copyright (C) 1998  Mark Crosbie  mcrosbie@best.com                         *
//*                                                                              *
//*  This program is free software; you can redistribute it and/or modify        *
//*  it under the terms of the GNU General Public License as published by        *
//*  the Free Software Foundation; either version 2 of the License, or           *
//*  (at your option) any later version.                                         *
//*                                                                              *
//*  This program is distributed in the hope that it will be useful,             *
//*  but WITHOUT ANY WARRANTY; without even the implied warranty of              *
//*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
//*  GNU General Public License for more details.                                *
//*                                                                              *
//*  You should have received a copy of the GNU General Public License           *
//*  along with this program; if not, write to the Free Software                 *
//*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                   *
//*                                                                              *
//*   void lcd_putc(char c)    send the byte c to the LCD                        *
//*   void lcd_clear(void)     clears the LCD screen                             *
//*   void lcd_home(void)      homes the LCD cursor                              *
//*   void lcd_puts()          print a string **UNIMPLEMENTED**                  *
//*   void lcd_printhex(char c) prints byte c as a 2 digit hex character         *
//*   void lcd_printdec(char c) prints byte c as a decimal digit string          *
//*   void lcd_printbin(char c) prints byte c as a binary string                 *
//*   void lcd_scroll_left(char n) scrolls LCD screen left n positions           *
//*   void lcd_scroll_right(char n) scrolls LCD screen right n positions         *
//*   void lcd_goto(char row, char col) goto a position                          *
//*                                                                              *
//* Constants used:                                                              *
//*    LCDPORT        port on the PIC the LCD is connected to                    *
//*    LCDPIN         what PIN the LCD serial input is connected to              *
//*                                                                              *
//* Libraries used:                                                              *
//*    pause() from the timeutil library.                                        *                            
//*                                                                              *
//*                                                                              *
//*     Future upgrades to this application include: Implementing large          *
//* multiline LCD displays, LCD graphics and formatting and displaying data on   * 
//* LCD displays.                                                                *                             
//*                                                                              *
//********************************************************************************
   
#ifndef LCD_LOADED  /* prevent multiple loading */
#define LCD_LOADED


#include "flags.h"                                    // Global flags used for MPSIM debugging 
#include "stdio.h"                                    // I/O support functions  
 

extern int LCD_Ready;									 // Set to true when LCD has been initialized

// Declare LCD function headers here
 
void InitializeLCD(void);                             // Initialize the LCD  
void WriteLCDCommand(byte Command);                   // Send command to the Hitachi HD44780 LCD Display controller
void WriteLCD(char c);                                // Write directly to the LCD
void lcd_putc(char c);                         			// Send the byte c to the LCD
void lcd_clear(void);     										//	clears the LCD screen
void lcd_home(void);      										// homes the LCD cursor
void lcd_puts(char *s);        							   // print a string **UNIMPLEMENTED**
void lcd_printhex(char c); 									// prints byte c as a 2 digit hex character
void lcd_printdec(char c); 									// prints byte c as a decimal digit string
void lcd_printbin(char c); 									//	prints byte c as a binary string
void lcd_scroll_left(char n); 								//	scrolls LCD screen left n positions
void lcd_scroll_right(char n); 								//	scrolls LCD screen right n positions
void lcd_goto(char row, char col); 							//	goto a position
void lcd_test_1(void);											// Test the LCD display drivers
void lcd_test_2(void);
void lcd_test_3(void);												 
void lcd_test_4(void);


#endif

