//********************************************************************************
//*                                                                              *
//* Description:                                                                 *
//*                                                                              *
//*     Name:  max7219.h                                                         *
//*   Author:  Daniel F. Ramirez                                                 *
//*     Date:  11/20/2000                                                        *
//*                                                                              *
//* Description:                                                                 *
//*         This is an embedded realtime LED Display application, that uses the  *
//* MAX7219 IC and a PIC18C452 micro-controller from Microchip Technology to     *
//* drive LED displays.  It employs the shiftout and pulsout functions to        *
//* communicate to the MAX7219 IC, and uses reverse, itoa and right_justify      *
//* functions to format data.  Currently it handles up to 2 MAX7219 drivers each *
//* capable of driving up to 5 LED digits.  The specs for the MAX7219 specify    *
//* that a maximum of 8 digits may be driven by it, but I have not been able to  *
//* get large 8 digits working at the same time, maybe because the large LED     *
//* digits are drawing too much current.  The floating point I/O routines        *
//* developed in this application are used for formatting and displaying         *
//* data on LED displays.  They may also be used for both the stdio.c and lcd.c  *    
//* applications to implement the printf %f floating point specifier.            *
//*                                                                              *
//*                                                                              *
//*     Future upgrades to this application include: Implementing large LED      *
//* displays; handling multiple MAX7219 devices; using the USART interface to    * 
//* make a serial LED display controller similar to the Scott Edwards serial LCD.*                                                  
//*                                                                              *
//********************************************************************************
   
#ifndef MAX7219_LOADED  /* prevent multiple loading */
#define MAX7219_LOADED


#include "flags.h"                                      // Global flags used for MPSIM debugging 
#include "stdio.h"                                        // I/O support functions  
 

// Declare function headers here

// Declare MAX7219 function headers here

void 				InitializeMAX7219(void);

void           MaxPrint(char *s); 

byte 				TranslateCharacter(char c);

//void         Reverse(char *s);

// Declare PC communication function headers here

void           right_justify(int size, char *s);
 

#endif

