//********************************************************************************
//*        Name:  myprintf.h                                                     *
//*      Author:  Daniel F. Ramirez                                              *
//*        Date:  12/26/2002                                                     *
//*                                                                              *
//*                                                                              * 
//* Description:                                                                 *
//*     This file contains my printf I/O functions used by the PIC "C" compiler  * 
//* Included are low level floating point conversion functions.                  *
//*                                                                              *
//*      Future upgrades to this application include: File I/O, I2c, and         *
//* Internet networking.                                                         *
//*                                                                              *
//*                                                                              *
//********************************************************************************
  
#ifndef MYPRINTF_LOADED  /* prevent multiple loading */
#define MYPRINTF_LOADED
       
#include "flags.h"                                      // Global flags used for MPSIM debugging 

#ifdef PIC18C252     
#include <p18c252.h>					// Register definitions for PIC18C252
#endif
  
#ifdef PIC18C452     
#include <p18c452.h>					// Register definitions for PIC18C452
#endif

#include <ctype.h>
#include <stdlib.h>
#include <string.h>
//#include <i2c.h>					// I2C library functions
//#include <pwm.h>                                        // PWM library functions

#ifdef PIC18C452   
#include <timers.h>                                     // Timer library functions
#include <portb.h>                                      // PORTB library functions                                 
#include <usart.h>                                      // USART library functions
#include <delays.h>                                     // High resolution delay functions
#endif

 
#define BufferSize		20			// Floating point buffer size

//********************************************************************************
// Function Prototypes
//********************************************************************************
 
// Floating point conversion functions

void Normalize(double *Value, int *Exponent);
unsigned long NormalizeFraction(double FractionalPart, int NumberOfDigits);

#ifdef SINGLE_PRECISION
void FloatToAscii(float Number, char * s);
void AsciiToFloat(float Number, char * s);
#endif
void DoubleToAscii(double Number, char * s);
void AsciiToDouble(double *Number, char * s);
int Member(char Ch, char * Set);

float GetFloat(void);
double GetDouble(void);

double log10(double x);
double pow10(int x);

#endif
